import React from 'react';
import { generate as generateColor } from '@ant-design/colors';

import type { AbstractNode, IconDefinition } from '@sensoro-design/icons-svg/lib/types';

export interface Attrs {
  [key: string]: string;
}

export const toHump = (name: string) => {
  return name.replace((/(-|:)\w/g), function(v){
    return v.substring(1).toUpperCase()
  })
}

export function normalizeAttrs(attrs: Attrs = {}): Attrs {
  return Object.keys(attrs)
    .reduce((acc: Attrs, key: string) => {
      const val = attrs[key];
      switch (key) {
        case 'class':
          acc.className = val;
          delete acc.class;
          break;
        default:
          acc[toHump(key)] = val;
      }
      return acc;
    }, {});
}

export function generate(
  node: AbstractNode,
  key: string,
  rootProps?: { [key: string]: any } | false,
): any {
  if (!rootProps) {
    return React.createElement(
      node.tag,
      { key, ...normalizeAttrs(node.attrs) },
      (node.children || []).map((child, index) => generate(child, `${key}-${node.tag}-${index}`)),
    );
  }
  return React.createElement(
    node.tag,
    {
      key,
      ...normalizeAttrs(node.attrs),
      ...rootProps,
    },
    (node.children || []).map((child, index) => generate(child, `${key}-${node.tag}-${index}`)),
  );
}

export function getSecondaryColor(primaryColor: string): string {
  // choose the second color
  return generateColor(primaryColor)[0];
}

export function isIconDefinition(target: any): target is IconDefinition {
  return (
    typeof target === 'object' &&
    typeof target.name === 'string' &&
    typeof target.theme === 'string' &&
    (typeof target.icon === 'object' || typeof target.icon === 'function')
  );
}

export const svgBaseProps = {
  width: '1em',
  height: '1em',
  fill: 'currentColor',
  'aria-hidden': 'true',
  focusable: 'false',
};

export function normalizeTwoToneColors(
  twoToneColor: string | [string, string] | undefined,
): string[] {
  if (!twoToneColor) {
    return [];
  }

  return Array.isArray(twoToneColor) ? twoToneColor : [twoToneColor];
}
